package com.iosoftware.ld22lma;

import java.io.*;
import java.util.*;
import javax.sound.sampled.*;

// Class for easy sound handling

// SoundManager.playOnce(String/InputStream/etc); <-- File is played once then discarded
// SoundManager.Sound sound = SoundManager.load(String/InputStream/etc); <-- File is loaded and can be played as often as wanted
// sound.play();

public class SoundManager {
	private static SoundManager inst;
	private Sound[] sounds;

	private SoundManager() {
		inst = this;
		sounds = new Sound[0];
		new Closer();
	}

	private static SoundManager get() {
		if (inst == null) {
			inst = new SoundManager();
		}
		return inst;
	}

	public static Sound load(String path) {
		/*
		 * if(!(new File(path).exists())) {
		 * System.out.println("Error: File '"+path+"' does not exist."); return
		 * null; }
		 */
		return load(Misc.getFileAsInputStream(path));
	}

	public static Sound load(InputStream in) {
		/*
		 * if (!(in instanceof BufferedInputStream)) { in = new
		 * BufferedInputStream(in); }
		 */
		return get().loadIntern(in);
	}

	public static Loop createLoop(Sound sound) {
		return new Loop(sound);
	}

	private Sound loadIntern(InputStream in) {
		try {
			/*
			 * AudioInputStream ais = AudioSystem.getAudioInputStream(in);
			 * AudioFormat af = ais.getFormat(); int size = (int)
			 * (af.getFrameSize() * ais.getFrameLength()); byte[] audio = new
			 * byte[size]; ais.read(audio, 0, size);
			 */
			ByteArrayOutputStream baos = new ByteArrayOutputStream();
			DataInputStream dis = new DataInputStream(in);
			byte[] buffer = new byte[1024];
			int read = 0;
			while ((read = dis.read(buffer)) >= 0) {
				baos.write(buffer, 0, read);
			}
			dis.close();
			byte[] audio = baos.toByteArray();
			Sound sound = new Sound();
			sound.audio = audio;
			synchronized (this) {
				int iC = sounds.length;
				Sound[] soundsNew = new Sound[iC + 1];
				for (int i = 0; i < iC; i++) {
					soundsNew[i] = sounds[i];
				}
				soundsNew[iC] = sound;
				sounds = soundsNew;
			}
			return sound;
		} catch (Exception ex) {
			ex.printStackTrace();
		}
		return null;
	}

	public static void playOnce(String path) {
		Sound sound = load(path);
		if (sound != null) {
			synchronized (sound) {
				sound.once = true;
				sound.play();
			}
		}
	}

	private class Closer extends Thread {
		private Closer() {
			setDaemon(true);
			setName("SoundManager");
			start();
		}

		public void run() {
			while (true) {
				synchronized (inst) {
					int iC = sounds.length;
					for (int i = 0; i < iC; i++) {
						Sound sound = sounds[i];
						synchronized (sound) {
							sound.tick();
							if (sound.canBeDeleted()) {
								Sound[] soundsNew = new Sound[iC - 1];
								int c = 0;
								for (int i2 = 0; i2 < iC; i2++) {
									if (sounds[i2] != sound) {
										soundsNew[c] = sounds[i2];
										c++;
									}
								}
								sounds = soundsNew;
								i--;
								iC--;
								continue;
							} else {
								if (!sound.once && sound.nextClip == null) {
									sound.loadNext();
								}
							}
						}
					}
				}
				Misc.sleep(10);
			}
		}
	}

	public static class Loop {
		private Clip myClip;
		private boolean exists;

		private Loop(Sound source) {
			exists = true;
			try {
				Clip clip = (Clip) AudioSystem.getClip();
				clip.open(AudioSystem
						.getAudioInputStream(new ByteArrayInputStream(
								source.audio)));
				myClip = clip;
			} catch (Exception ex) {
				ex.printStackTrace();
				exists = false;
			}
		}

		public void stopLoop() {
			myClip.stop();
		}

		public void startLoop() {
			myClip.loop(Clip.LOOP_CONTINUOUSLY);
		}

		public void endLoop() {
			myClip.stop();
			myClip.flush();
			myClip.close();
			exists = false;
		}

		public void setVolume(double volume) {
			FloatControl gainControl = (FloatControl) myClip
					.getControl(FloatControl.Type.MASTER_GAIN);
			float dB = (float) (Math.log(volume) / Math.log(10.0) * 20.0);
			gainControl.setValue(dB);

			BooleanControl muteControl = (BooleanControl) myClip
					.getControl(BooleanControl.Type.MUTE);
			muteControl.setValue(true);

			muteControl.setValue(false);
		}

		public void setBalance() {

		}
	}

	public static class Sound {
		private boolean once;
		private List<ClipWrapper> clips; // list of clips that are played atm
		private Clip nextClip;
		private byte[] audio;
		private boolean exists;

		private Sound() {
			once = false;
			clips = new ArrayList<ClipWrapper>();
			nextClip = null;
			audio = new byte[0];
			exists = true;
		}

		public void play() {
			synchronized (this) {
				if (nextClip == null) {
					loadNext();
				}
				Clip clip = nextClip;
				clip.start();
				clips.add(new ClipWrapper(clip, 100));
				nextClip = null;
			}
		}

		public void loop() {
			//
		}

		private void loadNext() {
			synchronized (this) {
				if (nextClip == null) {
					try {
						Clip clip = (Clip) AudioSystem.getClip();
						clip.open(AudioSystem
								.getAudioInputStream(new ByteArrayInputStream(
										audio)));
						nextClip = clip;
					} catch (Exception ex) {
						ex.printStackTrace();
					}
				}
			}
		}
		
		private void remove() {
			exists = false;
		}

		private void tick() {
			synchronized (this) {
				int iC = clips.size();
				for (int i = 0; i < iC; i++) {
					ClipWrapper wrap = clips.get(i);
					Clip clip = wrap.getClip();
					wrap.tick();
					if (wrap.canBeDeleted()) {
						clip.stop();
						clip.flush();
						clip.close();
						clips.remove(i);
						i--;
						iC--;
					}
				}
				if(once) {
					if(clips.size() == 0) {
						exists = false;
					}
				}
			}
		}

		private boolean canBeDeleted() {
			synchronized (this) {
				return !exists;
			}
		}

		private class ClipWrapper {
			private Clip clip;
			private int counter;

			private ClipWrapper(Clip clip, int counter) {
				this.clip = clip;
				this.counter = counter;
			}

			private Clip getClip() {
				return clip;
			}

			private void tick() {
				counter--;
				if (counter < 90 && !clip.isRunning()) {
					counter = 0;
				}
			}

			private boolean canBeDeleted() {
				return (counter <= 0) && !clip.isRunning();
			}
		}
	}
}
